<?php

namespace Authstack\OpenIDConnect\Model\Protocol\Flow;

use GuzzleHttp\Client;

abstract class AbstractFlow
{
    protected $httpClient;

    /**
     * @return Client
     */
    protected function getHttpClient(): Client
    {
        if(!$this->httpClient instanceof Client)
        {
            $this->httpClient = new Client;
        }

        return $this->httpClient;
    }
}